/*!
    \file    main.c
    \brief   led spark with systick, USART print and key example

    \version 2024-02-22, V2.1.0, firmware for GD32E23x
*/
#include "main.h"
#include <stdio.h>
#include "gd32e23x.h"
#include "systick.h"
#include "gd32e23x_libopt.h"
#include "RS485.h"
#include "LDC1612.h"

void led_config(void)
{
    rcu_periph_clock_enable(LED_RCU);

    gpio_mode_set(LED_PORT, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, LED_PIN);
    gpio_output_options_set(LED_PORT, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, LED_PIN);
    gpio_bit_write(LED_PORT, LED_PIN, SET);

    rcu_periph_clock_enable(LED_TIMER_RCU);
    timer_deinit(LED_TIMER);

    timer_parameter_struct timer_initpara;
    timer_struct_para_init(&timer_initpara);
    timer_initpara.prescaler = 799;
    timer_initpara.alignedmode = TIMER_COUNTER_EDGE;
    timer_initpara.counterdirection = TIMER_COUNTER_UP;
    timer_initpara.period = 999;
    timer_initpara.clockdivision = TIMER_CKDIV_DIV1;
    timer_init(LED_TIMER, &timer_initpara);

    timer_auto_reload_shadow_enable(LED_TIMER);
    timer_interrupt_enable(LED_TIMER, TIMER_INT_UP);

    timer_enable(LED_TIMER);

    nvic_irq_enable(LED_IRQ, 2);
}

void iicSendData(void)
{
    while (i2c_flag_get(I2C0, I2C_FLAG_I2CBSY));
    i2c_start_on_bus(I2C0);
    printf("1\r\n");
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND));
    i2c_master_addressing(I2C0, LDC1612_ADDR, I2C_TRANSMITTER);
    printf("2\r\n");
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND));
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    printf("3\r\n");
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE));
    i2c_data_transmit(I2C0, 0x7E);
    printf("4\r\n");
    i2c_start_on_bus(I2C0);
}

void iicReceiveData(void)
{
    uint16_t data[3] = {0};
    i2c_start_on_bus(I2C0);
    // i2c_stop_on_bus(I2C0);
    printf("5\r\n");
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND));
    i2c_master_addressing(I2C0, LDC1612_ADDR, I2C_RECEIVER);
    printf("6\r\n");
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND));
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    printf("7\r\n");
    i2c_ack_config(I2C0, I2C_ACK_DISABLE);
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE));
    data[0] = i2c_data_receive(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE));
    data[1] = i2c_data_receive(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE));
    data[2] = i2c_data_receive(I2C0);

    i2c_stop_on_bus(I2C0);
    i2c_ack_config(I2C0, I2C_ACK_ENABLE);

    printf("data[0] = %x\r\n", data[0]);
    printf("data[1] = %x\r\n", data[1]);
    printf("data[2] = %x\r\n", data[2]);
}

/*!
    \brief      main function
    \param[in]  none
    \param[out] none
    \retval     none
*/
int main(void)
{
    /* configure systick */
    systick_config();
    RS485_config();
    led_config();
    I2C_config();

    // delay_ms(5000);
    printf("\r\n");
    printf("XLSW-3DP-LDC1612! V0.0.1\r\n");
    printf("\r\n");

    // uint16_t data[3] = {0};

    // I2C _scan();

    delay_ms(200);

    // while (i2c_flag_get(I2C0, I2C_FLAG_I2CBSY));
    // i2c_start_on_bus(I2C0);
    // printf("1\r\n");
    // while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND));
    // i2c_master_addressing(I2C0, LDC1612_ADDR, I2C_TRANSMITTER);
    // printf("2\r\n");
    // while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND));
    // i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    // printf("3\r\n");
    // while (!i2c_flag_get(I2C0, I2C_FLAG_TBE));
    // i2c_data_transmit(I2C0, 0x7E);
    // printf("4\r\n");
    // i2c_start_on_bus(I2C0);
    // // i2c_stop_on_bus(I2C0);


    // delay_ms(100);
    // i2c_start_on_bus(I2C0);
    // delay_ms(100);
    // i2c_stop_on_bus(I2C0);


    // i2c_stop_on_bus(I2C0);
    //
    // i2c_ack_config(I2C0, I2C_ACK_ENABLE);

    iicSendData();
    iicReceiveData();

    // int result = 0;
    // result = LDC1612_IIC_read_16bits();
    // printf("result = %d\r\n", result);





    while(1){
        // printf("OK!!!\r\n");
        //
        // delay_ms(1000);
    }
}

/* retarget the C library printf function to the USART */
int _write (int fd, char *pBuffer, int size)
{
    for (int i = 0; i < size; i++)
    {
        usart_data_transmit(USART0, (uint8_t)pBuffer[i]);
        while(RESET == usart_flag_get(USART0, USART_FLAG_TBE));
    }
    return size;
}
